import argparse
import time
import typing

from ._crc32c import crc32c

DEFAULT_SIZE = 100 * 1024 * 1024
DEFAULT_ITERATIONS = 10


def run(size: int, iterations: int) -> typing.Tuple[float, int]:
    data = b" " * size
    start = time.monotonic()
    evaluations = 0
    while True:
        evaluations += iterations
        [crc32c(data) for _ in range(iterations)]
        duration = time.monotonic() - start
        if duration > 0:
            break
    return duration, evaluations


def main() -> None:

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-s",
        "--size",
        type=int,
        help=f"Amount of bytes to checksum, defaults to {DEFAULT_SIZE}",
        default=DEFAULT_SIZE,
    )
    parser.add_argument(
        "-i",
        "--iterations",
        type=int,
        help=f"Number of times the checksum should we run over the data, defaults to {DEFAULT_ITERATIONS}",
        default=DEFAULT_ITERATIONS,
    )

    options = parser.parse_args()
    duration, evaluations = run(options.size, options.iterations)
    size_mb = options.size / 1024 / 1024
    avg_speed_gbs = size_mb / 1024 * evaluations / duration
    print(
        f"crc32c ran at {avg_speed_gbs:.3f} [GB/s] when checksuming {size_mb:.3f} [MB] {evaluations} times"
    )


if __name__ == "__main__":
    main()



from typing_extensions import Buffer

big_endian: int
hardware_based: bool

def crc32(data: Buffer, value: int = 0, gil_release_mode: int = -1) -> int: ...
def crc32c(data: Buffer, value: int = 0, gil_release_mode: int = -1) -> int: ...



from __future__ import annotations

from typing import TYPE_CHECKING

from ._crc32c import crc32c

if TYPE_CHECKING:
    from typing_extensions import Buffer, Self


class CRC32CHash:
    """Wrapper class for crc32c. Tries to conform to the interface of `hashlib` classes."""

    @property
    def digest_size(self) -> int:
        """
        The size of the resulting hash in bytes.
        """
        return 4

    @property
    def block_size(self) -> int:
        """
        The internal block size of the hash algorithm in bytes.
        """
        return 1

    @property
    def name(self) -> str:
        """
        The canonical name of this hash,
        """
        return "crc32c"

    @property
    def checksum(self) -> int:
        """
        The checksum calculated so far. Not part of the hashlib interface.
        """
        return self._checksum

    def __init__(self, data: Buffer = b"", gil_release_mode: int = -1) -> None:
        """
        Initialise the hash object with an optional bytes-like object.
        Uses the given GIL release mode on each checksum calculation.
        """
        self._checksum = crc32c(data, gil_release_mode=gil_release_mode)
        self._gil_release_mode = gil_release_mode

    def update(self, data: Buffer) -> None:
        """
        Update the hash object with the bytes-like object.
        Repeated calls are equivalent to a single call with the concatenation of all the arguments:
        m.update(a); m.update(b) is equivalent to m.update(a+b).
        """
        self._checksum = crc32c(
            data, self._checksum, gil_release_mode=self._gil_release_mode
        )

    def digest(self) -> bytes:
        """
        Return the digest of the data passed to the update() method so far.
        This is a bytes object of size digest_size which may contain bytes in the whole range from 0 to 255.
        """
        return self._checksum.to_bytes(4, "big")

    def hexdigest(self) -> str:
        """
        Like digest() except the digest is returned as a string object of double length,
        containing only hexadecimal digits.
        This may be used to exchange the value safely in email or other non-binary environments.
        """
        return self.digest().hex()

    def copy(self) -> Self:
        """
        Return a copy (“clone”) of the hash object. This can be used to efficiently compute
        the digests of data sharing a common initial substring.
        """
        res = type(self)()
        res._checksum = self._checksum
        res._gil_release_mode = self._gil_release_mode
        return res



# Explicitly "import ... as" to make mypy --strict happy
from importlib.metadata import version

from ._cli import main
from ._crc32c import big_endian as big_endian
from ._crc32c import crc32 as crc32
from ._crc32c import crc32c as crc32c
from ._crc32c import hardware_based as hardware_based
from ._crc32hash import CRC32CHash as CRC32CHash

__version__ = version("crc32c")
del version
